@@ -38,9 +38,9 @@ except ImportError:
 
 
 # Initialise the constants.
-PYQT_VERSION_STR = "4.11.1"
+PYQT_VERSION_STR = "4.11.3"
 
-SIP_MIN_VERSION = '4.16'
+SIP_MIN_VERSION = '4.16.4'
 
 
 class ModuleMetadata:
@@ -117,7 +117,7 @@ QT5_MODULES = {
     'QtTest':           ModuleMetadata(qmake_QT=['testlib', 'widgets']),
     'QtWebKit':         ModuleMetadata(
                                 qmake_QT=['webkit', 'webkitwidgets',
-                                        'network']),
+                                        'network', 'printsupport']),
     'QtXml':            ModuleMetadata(qmake_QT=['xml', '-gui']),
     'QtXmlPatterns':    ModuleMetadata(
                                 qmake_QT=['xmlpatterns', '-gui', 'network'])
@@ -477,6 +477,7 @@ class TargetConfiguration:
         self.qmake_variables = []
         self.py_pylib_dir = ''
         self.py_pylib_lib = ''
+        self.py_pyshlib = ''
         self.pydbus_inc_dir = ''
         self.pydbus_module_dir = ''
         self.pyqt_disabled_features = []
@@ -548,6 +549,7 @@ class TargetConfiguration:
                 self.py_pylib_dir)
         self.py_pylib_lib = parser.get(section, 'py_pylib_lib',
                 self.py_pylib_lib)
+        self.py_pyshlib = parser.get(section, 'py_pyshlib', self.py_pyshlib)
 
         self.qt_shared = parser.getboolean(section, 'qt_shared',
                 self.qt_shared)
@@ -695,29 +697,41 @@ int main(int argc, char **argv)
             pylib_lib = 'python%d%d%s' % (py_major, py_minor, debug_suffix)
 
             pylib_dir = self.py_lib_dir
+
+            # Assume Python is a DLL on Windows.
+            pyshlib = pylib_lib
         else:
             abi = getattr(sys, 'abiflags', '')
             pylib_lib = 'python%d.%d%s' % (py_major, py_minor, abi)
+            pylib_dir = pyshlib = ''
 
             # Use distutils to get the additional configuration.
             from distutils.sysconfig import get_config_vars
             ducfg = get_config_vars()
 
-            exec_prefix = ducfg['exec_prefix']
-            multiarch = ducfg.get('MULTIARCH', '')
-            libdir = ducfg['LIBDIR']
+            config_args = ducfg.get('CONFIG_ARGS', '')
 
-            if glob.glob('%s/lib/libpython%d.%d*' % (exec_prefix, py_major, py_minor)):
-                pylib_dir = exec_prefix + '/lib'
-            elif multiarch != '' and glob.glob('%s/lib/%s/libpython%d.%d*' % (exec_prefix, multiarch, py_major, py_minor)):
-                pylib_dir = exec_prefix + '/lib/' + multiarch
-            elif glob.glob('%s/libpython%d.%d*' % (libdir, py_major, py_minor)):
-                pylib_dir = libdir
-            else:
-                pylib_dir = ''
+            dynamic_pylib = '--enable-shared' in config_args
+            if not dynamic_pylib:
+                dynamic_pylib = '--enable-framework' in config_args
+
+            if dynamic_pylib:
+                pyshlib = ducfg.get('LDLIBRARY', '')
+
+                exec_prefix = ducfg['exec_prefix']
+                multiarch = ducfg.get('MULTIARCH', '')
+                libdir = ducfg['LIBDIR']
+
+                if glob.glob('%s/lib/libpython%d.%d*' % (exec_prefix, py_major, py_minor)):
+                    pylib_dir = exec_prefix + '/lib'
+                elif multiarch != '' and glob.glob('%s/lib/%s/libpython%d.%d*' % (exec_prefix, multiarch, py_major, py_minor)):
+                    pylib_dir = exec_prefix + '/lib/' + multiarch
+                elif glob.glob('%s/libpython%d.%d*' % (libdir, py_major, py_minor)):
+                    pylib_dir = libdir
 
         self.py_pylib_dir = pylib_dir
         self.py_pylib_lib = pylib_lib
+        self.py_pyshlib = pyshlib
 
         # Apply sysroot where necessary.
         if self.sysroot != '':
@@ -798,16 +812,16 @@ int main(int argc, char **argv)
         if self.qmake_spec == '':
             self.qmake_spec = self.default_qmake_spec
 
-        # The binary OS/X Qt installer defaults to XCode.  If this is what we
-        # might have then use macx-clang (Qt v5) or macx-g++ (Qt v4).
-        if self.qmake_spec == 'macx-xcode':
-            if self.qt_version >= 0x050000:
-                # This will exist (and we can't check anyway).
-                self.qmake_spec = 'macx-clang'
-            else:
-                self.qmake_spec = 'macx-g++'
-
         if sys.platform == 'darwin':
+            # The binary OS/X Qt installer defaults to XCode.  If this is what
+            # we might have then use macx-clang (Qt v5) or macx-g++ (Qt v4).
+            if self.qmake_spec in ('macx-xcode', ''):
+                if self.qt_version >= 0x050000:
+                    # This will exist (and we can't check anyway).
+                    self.qmake_spec = 'macx-clang'
+                else:
+                    self.qmake_spec = 'macx-g++'
+
             # See if it is a framework.
             if os.access(os.path.join(qt_config.QT_INSTALL_LIBS, 'QtCore.framework'), os.F_OK):
                 self.qt_framework = True
@@ -1270,8 +1284,6 @@ def check_modules(target_config, verbose):
         check_module(target_config, verbose, 'QtDBus', 'qdbusconnection.h',
                     'QDBusConnection::systemBus()')
 
-    check_dbus(target_config, verbose)
-
 
 def generate_makefiles(target_config, verbose, no_timestamp, parts, tracing):
     """ Generate the makefiles to build everything.  target_config is the
@@ -1307,28 +1319,10 @@ def generate_makefiles(target_config, verbose, no_timestamp, parts, tracing):
             qpy_sources = []
             qpy_headers = []
 
-        if mname == 'QtCore':
-            qpy_sources.insert(0, 'qpycore_post_init.cpp')
-
         generate_sip_module_code(target_config, verbose, no_timestamp, parts,
                 tracing, mname, sip_flags, qpy_sources, qpy_headers)
         subdirs.append(mname)
 
-    # Embed the sip flags.
-    if 'QtCore' in target_config.pyqt_modules:
-        inform("Embedding sip flags...")
-
-        in_f = open(source_path('qpy', 'QtCore', 'qpycore_post_init.cpp.in'))
-        out_f = open_for_writing(
-                os.path.join('QtCore', 'qpycore_post_init.cpp'))
-
-        for line in in_f:
-            line = line.replace('@@PYQT_SIP_FLAGS@@', sip_flags)
-            out_f.write(line)
-
-        in_f.close()
-        out_f.close()
-
     # Generate the composite module.
     qtmod_sipdir = os.path.join('sip', 'Qt')
     mk_clean_dir(qtmod_sipdir)
@@ -1456,9 +1450,9 @@ INSTALLS += elementtree
         mode |= (stat.S_IXUSR | stat.S_IXGRP | stat.S_IXOTH)
         os.chmod(pyuic_wrapper, mode)
 
-    # Generate the top-level makefile.
-    inform("Generating the top-level Makefile...")
-    run_qmake(target_config, verbose, toplevel_pro)
+    # Generate the makefiles.
+    inform("Generating the Makefiles...")
+    run_qmake(target_config, verbose, toplevel_pro, recursive=True)
 
 
 def generate_plugin_makefile(target_config, verbose, plugin_dir, install_dir, plugin_name):
@@ -1471,9 +1465,9 @@ def generate_plugin_makefile(target_config, verbose, plugin_dir, install_dir, pl
     generated.
     """
 
-    # Check we have an interpreter library.
-    if target_config.py_pylib_dir == '':
-        inform("The %s plugin was disabled because the Python library couldn't be found." % plugin_name)
+    # Check we have a shared interpreter library.
+    if target_config.py_pyshlib == '':
+        inform("The %s plugin was disabled because a dynamic Python library couldn't be found." % plugin_name)
         return False
 
     # Create the qmake project file.
@@ -1481,18 +1475,19 @@ def generate_plugin_makefile(target_config, verbose, plugin_dir, install_dir, pl
 
     sp_plugin_dir = source_path(plugin_dir)
 
-    fin = open(os.path.join(sp_plugin_dir, 'python.pro-in'))
+    fin = open(os.path.join(sp_plugin_dir, '%s.pro-in' % plugin_dir))
     prj = fin.read()
     fin.close()
 
     prj = prj.replace('@QTCONFIG@',
             'debug' if target_config.debug else 'release')
-    prj = prj.replace('@PYINCDIR@', quote(target_config.py_inc_dir))
-    prj = prj.replace('@SIPINCDIR@', quote(target_config.sip_inc_dir))
+    prj = prj.replace('@PYINCDIR@', qmake_quote(target_config.py_inc_dir))
+    prj = prj.replace('@SIPINCDIR@', qmake_quote(target_config.sip_inc_dir))
     prj = prj.replace('@PYLINK@', target_config.get_pylib_link_arguments())
-    prj = prj.replace('@QTPLUGINDIR@', quote(install_dir))
+    prj = prj.replace('@PYSHLIB@', target_config.py_pyshlib)
+    prj = prj.replace('@QTPLUGINDIR@', qmake_quote(install_dir))
 
-    pro_name = os.path.join(plugin_dir, 'python.pro')
+    pro_name = os.path.join(plugin_dir, '%s.pro' % plugin_dir)
 
     mk_dir(plugin_dir)
     fout = open_for_writing(pro_name)
@@ -1508,18 +1503,14 @@ VPATH = %s
 
     fout.close()
 
-    # Create the makefile.
-    inform("Generating the %s plugin Makefile..." % plugin_name)
-
-    run_qmake(target_config, verbose, pro_name)
-
     return True
 
 
 def generate_application_makefile(target_config, verbose, src_dir):
     """ Create the makefile for a QtXml based application.  target_config is
     the target configuration.  verbose is set if the output is to be displayed.
-    src_dir is the name of the directory containing the source files.
+    src_dir is the name of the directory containing the source files.  (It is
+    assumed that it is not a path.)
     """
 
     mk_dir(src_dir)
@@ -1530,7 +1521,8 @@ def generate_application_makefile(target_config, verbose, src_dir):
 
     inform("Generating the .pro file for %s..." % app)
 
-    pro_lines = ['TEMPLATE = app', 'QT -= gui', 'QT += xml']
+    pro_lines = ['TARGET = %s' % app]
+    pro_lines.extend(['TEMPLATE = app', 'QT -= gui', 'QT += xml'])
     pro_lines.append(
             'CONFIG += warn_on %s' %
                     ('debug' if target_config.debug else 'release'))
@@ -1540,6 +1532,9 @@ def generate_application_makefile(target_config, verbose, src_dir):
     elif target_config.py_platform == 'darwin':
         pro_lines.append('CONFIG -= app_bundle')
 
+    # Work around QTBUG-39300.
+    pro_lines.append('CONFIG -= android_install')
+
     pro_lines.append('target.path = %s' % target_config.pyqt_bin_dir)
     pro_lines.append('INSTALLS += target')
 
@@ -1549,15 +1544,12 @@ def generate_application_makefile(target_config, verbose, src_dir):
 
     pro_lines.extend(pro_sources(sp_src_dir))
 
-    pro_name = os.path.join(src_dir, app + '.pro')
+    pro_name = os.path.join(src_dir, src_dir + '.pro')
 
     pro = open_for_writing(pro_name)
     pro.write('\n'.join(pro_lines))
     pro.close()
 
-    inform("Generating the Makefile for %s..." % app)
-    run_qmake(target_config, verbose, pro_name)
-
 
 def pro_sources(src_dir, other_headers=None, other_sources=None):
     """ Return the HEADERS and SOURCES variables for a .pro file by
@@ -1712,12 +1704,13 @@ def inform_user(target_config, sip_version):
         inform("PyQt4 will only be usable with signed interpreters.")
 
 
-def run_qmake(target_config, verbose, pro_name, makefile_name='', fatal=True):
+def run_qmake(target_config, verbose, pro_name, makefile_name='', fatal=True, recursive=False):
     """ Run qmake against a .pro file.  target_config is the target
     configuration.  verbose is set if the output is to be displayed.  pro_name
     is the name of the .pro file.  makefile_name is the name of the makefile
     to generate (and defaults to Makefile).  fatal is set if a qmake failure is
     considered a fatal error, otherwise False is returned if qmake fails.
+    recursive is set to use the -recursive flag.
     """
 
     # qmake doesn't behave consistently if it is not run from the directory
@@ -1743,6 +1736,9 @@ def run_qmake(target_config, verbose, pro_name, makefile_name='', fatal=True):
         args.append('-o')
         args.append(makefile_name)
 
+    if recursive:
+        args.append('-recursive')
+
     args.append(pro_file)
 
     run_command(' '.join(args), verbose)
@@ -2231,6 +2227,21 @@ def generate_sip_module_code(target_config, verbose, no_timestamp, parts, tracin
     if not os.access(os.path.join(mname, file_check), os.F_OK):
         error("Unable to create the C++ code.")
 
+    # Embed the sip flags.
+    if mname == 'QtCore':
+        inform("Embedding sip flags...")
+
+        in_f = open(source_path('qpy', 'QtCore', 'qpycore_post_init.cpp.in'))
+        out_f = open_for_writing(
+                os.path.join('QtCore', 'qpycore_post_init.cpp'))
+
+        for line in in_f:
+            line = line.replace('@@PYQT_SIP_FLAGS@@', sip_flags)
+            out_f.write(line)
+
+        in_f.close()
+        out_f.close()
+
     # Generate the makefile.
     includepath = libs = ''
     if target_config.vend_enabled:
@@ -2278,6 +2289,9 @@ def generate_module_makefile(target_config, verbose, mname, includepath='', libs
 
     pro_add_qt_dependencies(target_config, metadata, pro_lines)
 
+    # Work around QTBUG-39300.
+    pro_lines.append('CONFIG -= android_install')
+
     if not target_config.static:
         debug_suffix = get_win32_debug_suffix(target_config.debug)
         link = get_win32_python_library(target_config)
@@ -2394,9 +2408,6 @@ macx {
     pro.write('\n')
     pro.close()
 
-    inform("Generating the Makefile for the %s module..." % mname)
-    run_qmake(target_config, verbose, pro_name)
-
 
 def pro_add_qpy(mname, metadata, pro_lines):
     """ Add the qpy dependencies of a module to a .pro file.  mname is the
@@ -2439,10 +2450,11 @@ def fix_license(src_lfile, dst_lfile):
     f.close()
 
 
-def check_license(target_config, license_confirmed):
+def check_license(target_config, license_confirmed, introspecting):
     """ Handle the validation of the PyQt4 license.  target_config is the
     target configuration.  license_confirmed is set if the user has already
-    accepted the license.
+    accepted the license.  introspecting is set if the configuration is being
+    determined by introspection.
     """
 
     try:
@@ -2469,7 +2481,7 @@ def check_license(target_config, license_confirmed):
                             sys.platform))
 
     # Common checks.
-    if target_config.qt_licensee != 'Open Source' and ltype == 'GPL':
+    if introspecting and target_config.qt_licensee != 'Open Source' and ltype == 'GPL':
         error(
                 "This version of PyQt4 and the commercial version of Qt have "
                 "incompatible licenses.")
@@ -2694,7 +2706,8 @@ def main(argv):
     target_config.apply_post_options(opts)
 
     # Check the licenses are compatible.
-    check_license(target_config, opts.license_confirmed)
+    check_license(target_config, opts.license_confirmed,
+            (opts.config_file is None))
 
     # Check SIP is what we need.
     sip_version = check_sip(target_config)
@@ -2709,6 +2722,8 @@ def main(argv):
     if len(target_config.pyqt_modules) == 0:
         check_modules(target_config, opts.verbose)
 
+    check_dbus(target_config, opts.verbose)
+
     # Tell the user what's been found.
     inform_user(target_config, sip_version)
 
